part of gauges;

/// Represents the container to render the axis and its element
///
// ignore: must_be_immutable
class _AxisContainer extends StatelessWidget {
  _AxisContainer(this.gauge);

  /// Specifies the radial gauge
  SfRadialGauge gauge;

  ///list of widgets present in gauge
  List<Widget> _gaugeWidgets;

  /// Specifies the render box
  RenderBox _renderBox;

  /// Specifies the axis tap position
  Offset tapPosition;

  /// Method to update the pointer value
  void _updatePointerValue(BuildContext context, DragUpdateDetails details) {
    _renderBox = _renderBox ?? context.findRenderObject();
    if (details != null && details.globalPosition != null) {
      tapPosition = _renderBox.globalToLocal(details.globalPosition);
      for (num i = 0; i < gauge.axes.length; i++) {
        if (gauge.axes[i].pointers != null &&
            gauge.axes[i].pointers.isNotEmpty) {
          for (num j = 0; j < gauge.axes[i].pointers.length; j++) {
            final GaugePointer _pointer = gauge.axes[i].pointers[j];
            if (_pointer.enableDragging && _pointer._isDragStarted) {
              if (_pointer is RangePointer) {
                final RangePointer _rangePointer = _pointer;
                final double _centerX = _rangePointer._pointerRect.left +
                    _rangePointer._axis._radius;
                final double _centerY = _rangePointer._pointerRect.top +
                    _rangePointer._axis._radius;
                final double _x = tapPosition.dx - _centerX;
                final double _y = tapPosition.dy - _centerY;
                final bool _isInside = (_x * _x) + (_y * _y) <=
                    (_rangePointer._axis._radius * _rangePointer._axis._radius);
                if (_isInside) {
                  _pointer._updateDragValue(tapPosition.dx, tapPosition.dy);
                }
              } else if (_pointer._axis._axisRect.contains(tapPosition)) {
                _pointer._updateDragValue(tapPosition.dx, tapPosition.dy);
              }
            }
          }
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return Container(
        decoration: BoxDecoration(color: Colors.transparent),
        child: _getGaugeElements(context, constraints),
      );
    });
  }

  /// To initialize the gauge elements
  Widget _getGaugeElements(BuildContext context, BoxConstraints constraints) {
    _gaugeWidgets = <Widget>[];
    _calculateAxisElementPosition(context, constraints);
    _addGaugeElements(constraints, context);

    return GestureDetector(
        onPanStart: (DragStartDetails details) =>
            _checkPointerDragging(context, details),
        onPanUpdate: (DragUpdateDetails details) =>
            _updatePointerValue(context, details),
        onPanEnd: (DragEndDetails details) => _checkPointerIsDragged(),
        onTapDown: (TapDownDetails details) =>
            _checkIsAxisTapped(context, details),
        child: Container(
          decoration: BoxDecoration(color: Colors.transparent),
          child: Stack(children: _gaugeWidgets),
        ));
  }

  /// Method to check whether the axis is tapped
  void _checkIsAxisTapped(BuildContext context, TapDownDetails details) {
    _renderBox = _renderBox ?? context.findRenderObject();
    if (details != null && details.globalPosition != null) {
      if (gauge.axes.isNotEmpty) {
        for (num i = 0; i < gauge.axes.length; i++) {
          final RadialAxis _axis = gauge.axes[i];
          final Offset _offset =
              _renderBox.globalToLocal(details.globalPosition);
          if (_axis.onAxisTapped != null &&
              _axis._axisPath.getBounds().contains(_offset)) {
            _axis._getAngleFromOffset(_offset);
          }
        }
      }
    }
  }

  /// Method to check whether the axis pointer is dragging
  void _checkPointerDragging(BuildContext context, DragStartDetails details) {
    _renderBox = _renderBox ?? context.findRenderObject();
    if (details != null && details.globalPosition != null) {
      tapPosition = _renderBox.globalToLocal(details.globalPosition);
      for (num i = 0; i < gauge.axes.length; i++) {
        if (gauge.axes[i].pointers != null &&
            gauge.axes[i].pointers.isNotEmpty) {
          for (num j = 0; j < gauge.axes[i].pointers.length; j++) {
            final GaugePointer _pointer = gauge.axes[i].pointers[j];
            if (_pointer.enableDragging) {
              if (_pointer is RangePointer) {
                final RangePointer _rangePointer = _pointer;

                final Rect _pathRect = Rect.fromLTRB(
                    _rangePointer._arcPath.getBounds().left - 5,
                    _rangePointer._arcPath.getBounds().top - 5,
                    _rangePointer._arcPath.getBounds().right + 5,
                    _rangePointer._arcPath.getBounds().bottom + 5);
                if (_pathRect.contains(tapPosition)) {
                  _rangePointer._isDragStarted = true;
                  break;
                } else {
                  _rangePointer._isDragStarted = false;
                }
              } else {
                if (_pointer._pointerRect.contains(tapPosition)) {
                  _pointer._isDragStarted = true;
                  break;
                } else {
                  _pointer._isDragStarted = false;
                }
              }
            }
          }
        }
      }
    }
  }

  /// Method to ensure whether the pointer was dragged
  void _checkPointerIsDragged() {
    if (gauge.axes != null) {
      for (num i = 0; i < gauge.axes.length; i++) {
        if (gauge.axes[i].pointers != null &&
            gauge.axes[i].pointers.isNotEmpty) {
          for (num j = 0; j < gauge.axes[i].pointers.length; j++) {
            final GaugePointer _pointer = gauge.axes[i].pointers[j];
            if (_pointer.enableDragging) {
              if (_pointer._isDragStarted) {
                _pointer._createPointerDraggedArgs();
              }
              _pointer._isDragStarted = false;
            }
          }
        }
      }
    }
  }

  /// Calculates the axis position
  void _calculateAxisElementPosition(
      BuildContext context, BoxConstraints constraints) {
    if (gauge.axes != null && gauge.axes.isNotEmpty) {
      for (int i = 0; i < gauge.axes.length; i++) {
        gauge.axes[i]._gauge = gauge;
        gauge.axes[i]._calculateAxisRange(constraints);
      }
    }
  }

  /// Methods to add the gauge elements
  void _addGaugeElements(BoxConstraints constraints, BuildContext context) {
    if (gauge.axes != null && gauge.axes.isNotEmpty) {
      for (int i = 0; i < gauge.axes.length; i++) {
        final RadialAxis _axis = gauge.axes[i];
        _addAxis(_axis, constraints);

        if (_axis.ranges != null && _axis.ranges.isNotEmpty) {
          _addRange(_axis, constraints);
        }

        if (_axis.pointers != null && _axis.pointers.isNotEmpty) {
          for (num j = 0; j < _axis.pointers.length; j++) {
            if (_axis.pointers[j] is NeedlePointer) {
              final NeedlePointer _needlePointer = _axis.pointers[j];
              _addNeedlePointer(_axis, constraints, _needlePointer);
            } else if (_axis.pointers[j] is MarkerPointer) {
              final MarkerPointer _markerPointer = _axis.pointers[j];
              _addMarkerPointer(_axis, constraints, _markerPointer);
            } else if (_axis.pointers[j] is RangePointer) {
              final RangePointer _rangePointer = _axis.pointers[j];
              _addRangePointer(_axis, constraints, _rangePointer);
            }
          }
        }

        if (_axis.annotations != null && _axis.annotations.isNotEmpty) {
          _addAnnotation(_axis);
        }
      }
    }
    gauge._radialGaugeState.isLoaded = false;
  }

  /// Adds the axis
  void _addAxis(RadialAxis axis, BoxConstraints constraints) {
    _gaugeWidgets.add(Container(
      child: RepaintBoundary(
        child: CustomPaint(
            painter: _AxisPainter(gauge, axis, axis._needsRepaintAxis ?? true),
            size: Size(constraints.maxWidth, constraints.maxHeight)),
      ),
    ));
  }

  /// Adds the range pointer
  void _addRangePointer(
      RadialAxis axis, BoxConstraints constraints, RangePointer _rangePointer) {
    Animation<double> _pointerAnimation;
    Animation<double> _cornerAnimation;
    if (_rangePointer.enableAnimation && _rangePointer.animationDuration > 0) {
      gauge._radialGaugeState.animationController =
          AnimationController(vsync: gauge._radialGaugeState);
      gauge._radialGaugeState.animationController.duration =
          Duration(milliseconds: _rangePointer.animationDuration.toInt());
      gauge._radialGaugeState.animationController.addListener(() {
        gauge._radialGaugeState.pointerRepaintNotifier.value++;
      });
      gauge._radialGaugeState.controllerList
          .add(gauge._radialGaugeState.animationController);
      _rangePointer._animationEndValue = _rangePointer._getSweepAngle();
      final Curve _animationType =
          _getCurveAnimation(_rangePointer.animationType);
      final double _startValue = axis.isInversed ? 1 : 0;
      if (_rangePointer.cornerStyle == CornerStyle.bothCurve ||
          _rangePointer.cornerStyle == CornerStyle.endCurve) {
        _rangePointer._endCurveAnimationEndValue =
            _rangePointer._getEndCurveAngle();
        _cornerAnimation = Tween<double>(
                begin:
                    _rangePointer._endCurveAnimationStartValue ?? _startValue,
                end: _rangePointer._endCurveAnimationEndValue)
            .animate(CurvedAnimation(
                parent: gauge._radialGaugeState.animationController,
                curve: Interval(0.15, 1, curve: _animationType)));
      }

      _pointerAnimation = Tween<double>(
              begin: _rangePointer._animationStartValue ?? 0,
              end: _rangePointer._animationEndValue)
          .animate(CurvedAnimation(
              parent: gauge._radialGaugeState.animationController,
              curve: Interval(0.15, 1, curve: _animationType)));
    }

    _gaugeWidgets.add(Container(
      child: RepaintBoundary(
        child: CustomPaint(
            painter: _RangePointerPainter(
                gauge,
                axis,
                _rangePointer,
                _rangePointer._needsRepaintPointer ?? true,
                _pointerAnimation,
                _cornerAnimation,
                gauge._radialGaugeState.pointerRepaintNotifier),
            size: Size(constraints.maxWidth, constraints.maxHeight)),
      ),
    ));
    if (_rangePointer.enableAnimation && _rangePointer.animationDuration > 0) {
      gauge._radialGaugeState.animationController.forward(from: 0.0);
    }
  }

  /// Adds the needle pointer
  void _addNeedlePointer(RadialAxis axis, BoxConstraints constraints,
      NeedlePointer _needlePointer) {
    Animation<double> _pointerAnimation;
    if (_needlePointer.enableAnimation &&
        _needlePointer.animationDuration > 0) {
      gauge._radialGaugeState.animationController =
          AnimationController(vsync: gauge._radialGaugeState);
      gauge._radialGaugeState.animationController.duration =
          Duration(milliseconds: _needlePointer.animationDuration.toInt());
      gauge._radialGaugeState.animationController.addListener(() {
        gauge._radialGaugeState.pointerRepaintNotifier.value++;
      });
      gauge._radialGaugeState.controllerList
          .add(gauge._radialGaugeState.animationController);
      _needlePointer._animationEndValue = _needlePointer._getSweepAngle();
      final double _startValue = axis.isInversed ? 1 : 0;
      final double _endValue = axis.isInversed ? 0 : 1;
      double _actualValue = _needlePointer._animationStartValue ?? _startValue;
      _actualValue = _actualValue == _endValue ? _startValue : _actualValue;
      _pointerAnimation = Tween<double>(
              begin: _actualValue, end: _needlePointer._animationEndValue)
          .animate(CurvedAnimation(
              parent: gauge._radialGaugeState.animationController,
              curve: Interval(0.15, 1,
                  curve: _getCurveAnimation(_needlePointer.animationType))));
    }

    _gaugeWidgets.add(Container(
      child: RepaintBoundary(
        child: CustomPaint(
            painter: _NeedlePointerPainter(
                gauge,
                axis,
                _needlePointer,
                _needlePointer._needsRepaintPointer ?? true,
                _pointerAnimation,
                gauge._radialGaugeState.pointerRepaintNotifier),
            size: Size(constraints.maxWidth, constraints.maxHeight)),
      ),
    ));

    if (_needlePointer.enableAnimation &&
        _needlePointer.animationDuration > 0) {
      gauge._radialGaugeState.animationController.forward(from: 0.0);
    }
  }

  /// Adds the marker pointer
  void _addMarkerPointer(RadialAxis axis, BoxConstraints constraints,
      MarkerPointer _markerPointer) {
    Animation<double> _pointerAnimation;
    if (_markerPointer.enableAnimation &&
        _markerPointer.animationDuration > 0) {
      gauge._radialGaugeState.animationController =
          AnimationController(vsync: gauge._radialGaugeState);
      gauge._radialGaugeState.animationController.duration =
          Duration(milliseconds: _markerPointer.animationDuration.toInt());
      gauge._radialGaugeState.animationController.addListener(() {
        gauge._radialGaugeState.pointerRepaintNotifier.value++;
      });
      gauge._radialGaugeState.controllerList
          .add(gauge._radialGaugeState.animationController);
      _markerPointer._animationEndValue = _markerPointer._getSweepAngle();
      final double _startValue = axis.isInversed ? 1 : 0;
      _pointerAnimation = Tween<double>(
              begin: _markerPointer._animationStartValue ?? _startValue,
              end: _markerPointer._animationEndValue)
          .animate(CurvedAnimation(
              parent: gauge._radialGaugeState.animationController,
              curve: Interval(0.15, 1,
                  curve: _getCurveAnimation(_markerPointer.animationType))));
    }

    _gaugeWidgets.add(Container(
      child: RepaintBoundary(
        child: CustomPaint(
            painter: _MarkerPointerPainter(
                gauge,
                axis,
                _markerPointer,
                _markerPointer._needsRepaintPointer ?? true,
                _pointerAnimation,
                gauge._radialGaugeState.pointerRepaintNotifier),
            size: Size(constraints.maxWidth, constraints.maxHeight)),
      ),
    ));

    if (_markerPointer.enableAnimation &&
        _markerPointer.animationDuration > 0) {
      gauge._radialGaugeState.animationController.forward(from: 0.0);
    }
  }

  /// Adds the axis range
  void _addRange(RadialAxis axis, BoxConstraints constraints) {
    for (num k = 0; k < axis.ranges.length; k++) {
      _gaugeWidgets.add(Container(
        child: RepaintBoundary(
          child: CustomPaint(
              painter: _RangePainter(
                  gauge,
                  axis,
                  axis.ranges[k],
                  axis.ranges[k]._needsRepaintRange ?? true,
                  gauge._radialGaugeState.rangeRepaintNotifier),
              size: Size(constraints.maxWidth, constraints.maxHeight)),
        ),
      ));
    }
  }

  /// Adds the axis annotation
  void _addAnnotation(RadialAxis axis) {
    for (num j = 0; j < axis.annotations.length; j++) {
      final GaugeAnnotation _annotation = axis.annotations[j];
      _gaugeWidgets
          .add(_AnnotationRenderer(key: GlobalKey(), annotation: _annotation));
    }
  }

  /// Method returns the curve animation function based on the animation type
  Curve _getCurveAnimation(AnimationType type) {
    Curve _curve = Curves.linear;
    switch (type) {
      case AnimationType.bounceOut:
        _curve = Curves.bounceOut;
        break;
      case AnimationType.ease:
        _curve = Curves.ease;
        break;
      case AnimationType.easeInCirc:
        _curve = Curves.easeInCirc;
        break;
      case AnimationType.easeOutBack:
        _curve = Curves.easeOutBack;
        break;
      case AnimationType.elasticOut:
        _curve = Curves.elasticOut;
        break;
      case AnimationType.linear:
        _curve = Curves.linear;
        break;
      case AnimationType.slowMiddle:
        _curve = Curves.slowMiddle;
        break;
    }
    return _curve;
  }
}
