part of gauges;

/// Represents the painter to draw marker
class _MarkerPointerPainter extends CustomPainter {
  _MarkerPointerPainter(
    this._gauge,
    this._axis,
    this._markerPointer,
    this._isRepaint,
    this._pointerAnimation,
    ValueNotifier<num> notifier,
  ) : super(repaint: notifier);

  /// Specifies the circular gauge
  final SfRadialGauge _gauge;

  /// Specifies whether to repaint the series
  final bool _isRepaint;

  /// Specifies the axis of the painter
  final RadialAxis _axis;

  /// Specifies the marker pointer
  final MarkerPointer _markerPointer;

  /// Specifies the pointer animation
  final Animation<double> _pointerAnimation;

  Offset _offset;

  @override
  void paint(Canvas canvas, Size size) {
    double _markerAngle = 0;

    if (_markerPointer.enableAnimation &&
        _markerPointer.animationDuration > 0 &&
        _markerPointer._needsAnimate) {
      _markerAngle =
          (_axis._sweepAngle * _pointerAnimation.value) + _axis.startAngle;
      _offset =
          _markerPointer._calculateMarkerOffset(_degreeToRadian(_markerAngle));
    } else {
      _offset = _markerPointer._offset;
      _markerAngle = _markerPointer._angle;
    }

    final double _animationValue = (_markerPointer.enableAnimation &&
            _markerPointer.animationDuration > 0 &&
            _markerPointer._needsAnimate)
        ? _pointerAnimation.value
        : null;
    _markerPointer.drawPointer(
        canvas, _animationValue, _offset, _offset, _markerAngle);

    if (_markerPointer.enableAnimation &&
        _markerPointer.animationDuration > 0 &&
        _markerAngle ==
            _axis._sweepAngle * _markerPointer._animationEndValue +
                _axis.startAngle) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        _gauge._radialGaugeState._updateAnimation(_markerPointer);
      });
    }
  }

  /// Renders the MarkerShape.circle

  @override
  bool shouldRepaint(_MarkerPointerPainter oldDelegate) => _isRepaint;
}
