part of gauges;

/// Custom painter to paint gauge axis
class _AxisPainter extends CustomPainter {
  _AxisPainter(this._gauge, this._axis, this._isRepaint) : super();

  /// Specifies the circular gauge
  final SfRadialGauge _gauge;

  /// Specifies whether to repaint the series
  final bool _isRepaint;

  /// Specifies the axis of the painter
  final RadialAxis _axis;

  @override
  void paint(Canvas canvas, Size size) {
    if (_axis.showAxisLine && _axis._actualAxisWidth > 0) {
      _drawAxisLine(canvas);
    }

    if (_axis.showTicks) {
      _drawMajorTicks(canvas);
      _drawMinorTicks(canvas);
    }

    if (_axis.showLabels) {
      _drawAxisLabels(canvas);
    }
  }

  /// Method to draw the axis line
  void _drawAxisLine(Canvas canvas) {
    if (_axis.axisLineStyle.cornerStyle == CornerStyle.bothFlat) {
      _drawAxisPath(canvas, _axis._startRadian, _axis._endRadian);
    } else {
      _drawAxisPath(
        canvas,
        _axis._startCornerRadian,
        _axis._sweepCornerRadian,
      );
      _drawAxisCorner(canvas);
    }
  }

  /// To draw the axis corner
  void _drawAxisCorner(Canvas canvas) {
    final Paint _cornerPaint = Paint()
      ..color = _axis.axisLineStyle.color ?? _gauge._gaugeTheme.axisLineColor;
    if (_axis.axisLineStyle.cornerStyle != CornerStyle.bothCurve) {
      canvas.drawCircle(
          _axis._cornerOffset, _axis._actualAxisWidth / 2, _cornerPaint);
    } else {
      canvas.drawCircle(
          _axis._cornerStartOffset, _axis._actualAxisWidth / 2, _cornerPaint);
      canvas.drawCircle(
          _axis._cornerEndOffset, _axis._actualAxisWidth / 2, _cornerPaint);
    }
  }

  /// Method to draw axis line
  void _drawAxisPath(Canvas canvas, double _startRadian, double _endRadian) {
    final Paint paint = Paint()
      ..color = _axis.axisLineStyle.color ?? _gauge._gaugeTheme.axisLineColor
      ..style = PaintingStyle.stroke
      ..strokeWidth = _axis._actualAxisWidth;
    if (_axis.axisLineStyle.dashArray != null &&
        _axis.axisLineStyle.dashArray.isNotEmpty &&
        _axis.axisLineStyle.dashArray.length > 1 &&
        _axis.axisLineStyle.dashArray[0] > 0 &&
        _axis.axisLineStyle.dashArray[1] > 0) {
      final Path path = Path();
      path.addArc(_axis._axisRect, _startRadian, _endRadian);
      canvas.drawPath(
          _dashPath(path,
              dashArray:
                  _CircularIntervalList<double>(_axis.axisLineStyle.dashArray)),
          paint);
    } else {
      canvas.drawArc(_axis._axisRect, _startRadian, _endRadian, false, paint);
    }
  }

  /// Method to draw the major ticks
  void _drawMajorTicks(Canvas canvas) {
    if (_axis._actualMajorTickLength > 0 &&
        _axis.majorTickStyle.thickness != null &&
        _axis.majorTickStyle.thickness > 0) {
      final Paint _tickPaint = Paint()
        ..style = PaintingStyle.stroke
        ..strokeWidth = _axis.majorTickStyle.thickness;
      for (num i = 0; i < _axis._majorTickOffsets.length; i++) {
        final _TickOffset _tickOffset = _axis._majorTickOffsets[i];
        if (!(i == 0 && _axis._sweepAngle == 360)) {
          _tickPaint.color = _axis.useRangeColorForAxis
              ? _axis._getRangeColor(_tickOffset.value) ??
                  _axis.majorTickStyle.color ??
                  _gauge._gaugeTheme.majorTickColor
              : _axis.majorTickStyle.color ?? _gauge._gaugeTheme.majorTickColor;

          if (_axis.majorTickStyle.dashArray != null &&
              _axis.majorTickStyle.dashArray.isNotEmpty) {
            final Path path = Path()
              ..moveTo(_tickOffset.startPoint.dx, _tickOffset.startPoint.dy)
              ..lineTo(_tickOffset.endPoint.dx, _tickOffset.endPoint.dy);
            canvas.drawPath(
                _dashPath(path,
                    dashArray: _CircularIntervalList<double>(
                        _axis.majorTickStyle.dashArray)),
                _tickPaint);
          } else {
            if ((i == _axis._majorTickOffsets.length - 1) &&
                _axis._sweepAngle == 360) {
              final double _x1 = (_axis._majorTickOffsets[0].startPoint.dx +
                      _axis._majorTickOffsets[i].startPoint.dx) /
                  2;
              final double _y1 = (_axis._majorTickOffsets[0].startPoint.dy +
                      _axis._majorTickOffsets[i].startPoint.dy) /
                  2;
              final double _x2 = (_axis._majorTickOffsets[0].endPoint.dx +
                      _axis._majorTickOffsets[i].endPoint.dx) /
                  2;
              final double _y2 = (_axis._majorTickOffsets[0].endPoint.dy +
                      _axis._majorTickOffsets[i].endPoint.dy) /
                  2;
              canvas.drawLine(Offset(_x1, _y1), Offset(_x2, _y2), _tickPaint);
            } else {
              canvas.drawLine(
                  _tickOffset.startPoint, _tickOffset.endPoint, _tickPaint);
            }
          }
        }
      }
    }
  }

  /// Method to draw the mior ticks
  void _drawMinorTicks(Canvas canvas) {
    if (_axis._actualMinorTickLength > 0 &&
        _axis.minorTickStyle.thickness != null &&
        _axis.minorTickStyle.thickness > 0) {
      final Paint _tickPaint = Paint()
        ..style = PaintingStyle.stroke
        ..strokeWidth = _axis.minorTickStyle.thickness;
      for (num i = 0; i < _axis._minorTickOffsets.length; i++) {
        final _TickOffset _tickOffset = _axis._minorTickOffsets[i];
        _tickPaint.color = _axis.useRangeColorForAxis
            ? _axis._getRangeColor(_tickOffset.value) ??
                _axis.minorTickStyle.color ??
                _gauge._gaugeTheme.minorTickColor
            : _axis.minorTickStyle.color ?? _gauge._gaugeTheme.minorTickColor;
        if (_axis.majorTickStyle.dashArray != null &&
            _axis.minorTickStyle.dashArray.isNotEmpty) {
          final Path path = Path()
            ..moveTo(_tickOffset.startPoint.dx, _tickOffset.startPoint.dy)
            ..lineTo(_tickOffset.endPoint.dx, _tickOffset.endPoint.dy);
          canvas.drawPath(
              _dashPath(path,
                  dashArray: _CircularIntervalList<double>(
                      _axis.minorTickStyle.dashArray)),
              _tickPaint);
        } else {
          canvas.drawLine(
              _tickOffset.startPoint, _tickOffset.endPoint, _tickPaint);
        }
      }
    }
  }

  /// Method to draw the axis labels
  void _drawAxisLabels(Canvas canvas) {
    for (num i = 0; i < _axis._axisLabels.length; i++) {
      if (!((i == 0 && !_axis.showFirstLabel) ||
          (i == _axis._axisLabels.length - 1 && !_axis.showLastLabel))) {
        final CircularAxisLabel label = _axis._axisLabels[i];
        final Color _labelColor =
            label.labelStyle.color ?? _gauge._gaugeTheme.axisLabelColor;
        final TextSpan span = TextSpan(
            text: label.text,
            style: TextStyle(
                color: _axis.ranges != null &&
                        _axis.ranges.isNotEmpty &&
                        _axis.useRangeColorForAxis
                    ? _axis._getRangeColor(label.value) ?? _labelColor
                    : _labelColor,
                fontSize: label.labelStyle.fontSize,
                fontFamily: label.labelStyle.fontFamily,
                fontStyle: label.labelStyle.fontStyle,
                fontWeight: label.labelStyle.fontWeight));

        final TextPainter textPainter = TextPainter(
            text: span,
            textDirection: TextDirection.ltr,
            textAlign: TextAlign.center);
        textPainter.layout();

        if (_axis.needsRotateLabels || label._needsRotateLabel) {
          canvas.save();
          canvas.translate(label.position.dx, label.position.dy);
          canvas.rotate(_degreeToRadian(label.angle));
          canvas.scale(-1);
          textPainter.paint(canvas,
              Offset(-label.labelSize.width / 2, -label.labelSize.height / 2));
          canvas.restore();
        } else {
          textPainter.paint(
              canvas,
              Offset(label.position.dx - label.labelSize.width / 2,
                  label.position.dy - label.labelSize.height / 2));
        }
      }
    }
  }

  @override
  bool shouldRepaint(_AxisPainter oldDelegate) => _isRepaint;
}
