part of gauges;

/// Represents the painter to render axis range
class _RangePainter extends CustomPainter {
  /// Creates the range painter
  _RangePainter(this._gauge, this._axis, this._range, this._isRepaint,
      ValueNotifier<num> notifier)
      : super(repaint: notifier);

  /// Specifies the circular gauge
  final SfRadialGauge _gauge;

  /// Specifies the gauge axis
  final RadialAxis _axis;

  /// Specifies the needle pointer
  final GaugeRange _range;

  /// Specifies whether to redraw the pointer
  final bool _isRepaint;

  @override
  bool shouldRepaint(_RangePainter oldDelegate) => _isRepaint;

  @override
  void paint(Canvas canvas, Size size) {
    Paint paint;
    final Path path = Path();
    if (_range._actualStartValue != _range._actualEndValue) {
      if (_range._rangeRect == null) {
        paint = Paint()
          ..style = PaintingStyle.fill
          ..color = _range.color ?? _gauge._gaugeTheme.rangeColor;
        path.arcTo(
            _range._outerArc._arcRect,
            _degreeToRadian(_range._outerArc._startAngle),
            _degreeToRadian(_range._outerArcSweepAngle),
            false);
        path.arcTo(
            _range._innerArc._arcRect,
            _degreeToRadian(_range._innerArc._endAngle),
            _degreeToRadian(_range._innerArcSweepAngle),
            false);
        canvas.drawPath(path, paint);
      } else {
        paint = Paint()
          ..strokeWidth = _range._thickness
          ..style = PaintingStyle.stroke
          ..color = _range.color ?? _gauge._gaugeTheme.rangeColor;

        canvas.drawArc(_range._rangeRect, _range._rangeStartRadian,
            _range._rangeEndRadian, false, paint);
      }
    }

    if (_range.label != null) {
      _renderRangeText(canvas);
    }
  }

  /// Renders the range text
  void _renderRangeText(Canvas canvas) {
    final Color _color =
        _range.color != null ? _range.color : _gauge._gaugeTheme.rangeColor;
    final TextSpan span = TextSpan(
        text: _range.label,
        style: TextStyle(
            color: _range.labelStyle.color ?? _getSaturationColor(_color),
            fontSize: _range.labelStyle.fontSize,
            fontFamily: _range.labelStyle.fontFamily,
            fontStyle: _range.labelStyle.fontStyle,
            fontWeight: _range.labelStyle.fontWeight));
    final TextPainter textPainter = TextPainter(
        text: span,
        textDirection: TextDirection.ltr,
        textAlign: TextAlign.center);
    textPainter.layout();
    canvas.save();
    canvas.translate(_range._labelPosition.dx, _range._labelPosition.dy);
    canvas.rotate(_degreeToRadian(_range._labelAngle));
    canvas.scale(-1);
    textPainter.paint(canvas,
        Offset(-_range._labelSize.width / 2, -_range._labelSize.height / 2));
    canvas.restore();
  }
}
