part of gauges;

/// Represents the painter to render the range pointer
class _RangePointerPainter extends CustomPainter {
  /// Creates the range pointer
  _RangePointerPainter(
      this._gauge,
      this._axis,
      this._rangePointer,
      this._isRepaint,
      this._pointerAnimation,
      this._curveAnimation,
      ValueNotifier<num> notifier)
      : super(repaint: notifier);

  /// Specifies the circular gauge
  final SfRadialGauge _gauge;

  /// Specifies the gauge axis
  final RadialAxis _axis;

  /// Specifies the needle pointer
  final RangePointer _rangePointer;

  /// Specifies whether to redraw the pointer
  final bool _isRepaint;

  /// Specifies the pointer animation
  final Animation<double> _pointerAnimation;

  /// Specifies the corner animation value
  final Animation<double> _curveAnimation;

  @override
  void paint(Canvas canvas, Size size) {
    final double _sweepRadian = _rangePointer.enableAnimation &&
            _rangePointer._needsAnimate &&
            _rangePointer.animationDuration > 0
        ? _degreeToRadian(_axis._sweepAngle * _pointerAnimation.value) ??
            _rangePointer._sweepCornerRadian
        : _rangePointer._sweepCornerRadian;
    final Paint paint = Paint()
      ..color = _rangePointer.color ?? _gauge._gaugeTheme.rangePointerColor
      ..strokeWidth = _rangePointer._actualRangeThickness
      ..style = PaintingStyle.stroke;

    canvas.save();
    final Path path = Path()
      ..arcTo(_rangePointer._arcRect, _rangePointer._startCornerRadian,
          _sweepRadian, false);

    canvas.drawPath(path, paint);
    if (_rangePointer.cornerStyle != CornerStyle.bothFlat) {
      final Paint _cornerPaint = Paint()
        ..color = _rangePointer.color ?? _gauge._gaugeTheme.rangePointerColor;
      if (_rangePointer.cornerStyle == CornerStyle.startCurve) {
        canvas.drawCircle(_rangePointer._cornerOffset,
            _rangePointer._actualRangeThickness / 2, _cornerPaint);
      } else if (_rangePointer.cornerStyle == CornerStyle.endCurve) {
        final Offset _offset = _getEndCurveOffset();
        canvas.drawCircle(
            _offset, _rangePointer._actualRangeThickness / 2, _cornerPaint);
      } else {
        final Offset _offset = _getEndCurveOffset();
        canvas.drawCircle(
            _offset, _rangePointer._actualRangeThickness / 2, _cornerPaint);
        canvas.drawCircle(_rangePointer._cornerStartOffset,
            _rangePointer._actualRangeThickness / 2, _cornerPaint);
      }
    }

    canvas.restore();
    if (_rangePointer.enableAnimation &&
        _rangePointer._needsAnimate &&
        _sweepRadian == _rangePointer._sweepCornerRadian) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        _gauge._radialGaugeState._updateAnimation(_rangePointer);
      });
    }
  }

  Offset _getEndCurveOffset() {
    Offset _endOffset;
    if (_rangePointer.enableAnimation &&
        _rangePointer.animationDuration > 0 &&
        _rangePointer._needsAnimate) {
      final double _endAngle =
          _curveAnimation.value * _axis._sweepAngle + _axis.startAngle;
      _endOffset = _rangePointer._calculateArcPoint(
          _rangePointer._midX,
          _rangePointer._midY,
          _rangePointer._cornerCenter,
          _degreeToRadian(_endAngle));
    } else {
      _endOffset = _rangePointer._cornerEndOffset;
    }

    return _endOffset;
  }

  @override
  bool shouldRepaint(_RangePointerPainter oldDelegate) => _isRepaint;
}
